#Copyright 2017 Tim Wentlau.
#Distributed under the MIT License. See LICENSE in root of project.


from kervi.spine import Spine

class Settings(object):
    """
    Class that persists settings to the Kervi database.

    :param group:
            To avoid name clash with other settings in the Kervi application
             enter name to group your settings under.

    :type group: ``str``

    """
    def __init__(self, settings_group=None):
        self.group = settings_group
        self.spine = Spine()

    def store_value(self, name, value):
        """Store a value to DB"""
        self.spine.send_command("storeSetting", self.group, name, value)


    def retrieve_value(self, name, default_value=None):
        """Retrieve a value from DB"""
        value = self.spine.send_query("retrieveSetting", self.group, name, processes=["kervi-main"])
        if value is None:
            return default_value
        elif isinstance(value, list) and len(value) == 0:
            return default_value
        elif isinstance(default_value, int):
            return int(value)
        elif isinstance(default_value, float):
            return float(value)
        else:
            return value



#Copyright 2017 Tim Wentlau.
#Distributed under the MIT License. See LICENSE in root of project.

import time

class OneWire(object):
    def __init__(self, address):
        self._address = address

    @property
    def address(self):
        return self._address

    def read(self):
        lines = self.read_raw()
        while lines[0].strip()[-3:] != 'YES':
            time.sleep(0.2)
            lines = self.read_raw()
        return lines[1]

    def read_raw(self):
        """Read raw data from one wire device"""
        raise NotImplementedError



""" Super simple template engine copied from Eric BREHAULT on github. """

    # import string

    # class SuperFormatter(string.Formatter):
    #     """World's simplest Template engine."""

    #     def format_field(self, value, spec):
    #         if spec.startswith('repeat'):
    #             template = spec.partition(':')[-1]
    #             if type(value) is dict:
    #                 value = value.items()
    #             return ''.join([template.format(item=item) for item in value])
    #         elif spec == 'call':
    #             return value()
    #         elif spec.startswith('if'):
    #             return (value and spec.partition(':')[-1]) or ''
    #         else:
    #             return super(SuperFormatter, self).format_field(value, spec)

class SuperFormatter():
    def __init__(self):
        pass

    def format(self, template, **kwargs):
        return template.format_map(kwargs)





#Copyright 2016 Tim Wentlau.
#Distributed under the MIT License. See LICENSE in root of project.

"""
Module that holds log functionality.
In general you dont need to include this in your kervi code as the spine class holds a refference
to a fully configured log.
"""

import logging
import os

class BraceMessage(object):
    def __init__(self, fmt, *args):
        self.fmt = fmt
        self.args = args

    def __str__(self):
        return self.fmt.format(*self.args)

class KerviLog(object):
    def __init__(self, name):
        self.logger = logging.getLogger(name)

    def info(self, message, *args):
        self.logger.debug(BraceMessage(message, *args))

    def debug(self, message, *args):
        if args:
            self.logger.debug(BraceMessage(message, *args))
        else:
            self.logger.debug(message)

    def error(self, message, *args):
        self.logger.error(BraceMessage(message, *args))

    def exception(self, message, *args):
        self.logger.exception(BraceMessage(message, *args))

    def fatal(self, message, *args):
        self.logger.fatal(BraceMessage(message, *args))

def init_process_logging(process_name, config):
    logger = logging.getLogger(process_name)
    if config.level == "warning":
        logger.setLevel(logging.WARNING)
    elif config.level == "debug":
        logger.setLevel(logging.DEBUG)

    if config.resetLog:
        try:
            os.remove(config.file)
        except:
            pass
    file_handler = logging.FileHandler(config.file)
    file_handler.setLevel(logging.DEBUG)
    stream_handler = logging.StreamHandler()
    stream_handler.setLevel(logging.ERROR)
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    file_handler.setFormatter(formatter)
    stream_handler.setFormatter(formatter)
    logger.addHandler(file_handler)
    logger.addHandler(stream_handler)



#Copyright 2017 Tim Wentlau.
#Distributed under the MIT License. See LICENSE in root of project.

import sys
from kervi.config.configuration import _Configuration

this = sys.modules[__name__]

this.Configuration = _Configuration()
this.Text = _Configuration()

def load(**kwargs):
    this.Configuration._load(**kwargs)
    this.Text._load(config_base = this.Configuration.texts.as_dict())
    return this.Configuration



#Copyright 2017 Tim Wentlau.
#Distributed under the MIT License. See LICENSE in root of project.

import importlib
import json
import collections
import os

def _deep_update(d, u):
    """Update a nested dictionary or similar mapping.
    Modify ``source`` in place.
    """

    for k, v in u.items():
        if isinstance(v, collections.Mapping):
            r = _deep_update(d.get(k, {}), v)
            d[k] = r
        else:
            d[k] = u[k]
    return d

class _KerviConfig:
    def __init__(self):
        self._keys = []

    def __getitem__(self, i):
        if type(i) is int:
            attr = self._keys[i]
            return getattr(self, attr)
        else:
            return getattr(self, i)


    def __len__(self):
        return len(self._keys)

    def get(self, name, default_value=None):
            if not name:
                return default_value
            elif hasattr(self, name):
                return getattr(self, name)
            else:
                return default_value
    def as_dict(self):
        d = dict()
        for key in self._keys:
            v = getattr(self, key)
            if isinstance(v, _KerviConfig):
                d[key] = v.as_dict()
            else:
                d[key] = getattr(self, key)

        return d
        
    def to_json(self):
        return json.dumps(self.as_dict())

    @property
    def keys(self):
        return self._keys

class _Configuration:
    instance = None
    class __ConfigClass(_KerviConfig):
        def __init__(self):
            super()
            self._keys = []
            self._config_path = None
            self._json_data = None
            self._user_config = None
            self._config = None
            self._config_base = None
            self._is_loaded = False

        def _load(self, **kwargs):
            self._config_path = kwargs.get("config_file", None)
            self._json_data = kwargs.get("json_data", None)
            self._user_data = kwargs.get("config_user", {})
            self._config_base = kwargs.get("config_base", {})

            if self._json_data:
                self._user_config = json.loads(self._json_data)
            elif not self._user_config and self._config_path:
                try:
                    if os.path.isfile(self._config_path):
                        print("use config:", self._config_path)
                        config_data = ""

                        with open(self._config_path, "r") as config_file:
                            for line in config_file:
                                line_trim = line.lstrip()
                                if not line_trim.startswith("#"):
                                    config_data += line
                                else:
                                    config_data += "\n"
                        if config_data:
                            self._user_config = json.loads(config_data)
                        #print(self._user_config)
                except Exception as ex:
                    print("error in config file", ex)
                    pass

            self._config = self._config_base
            if self._user_config:
                self._config = _deep_update(self._config, self._user_config)
            if self._user_data:
                self._config = _deep_update(self._config, self._user_data)

            #self.__dict__.update(config)
            self._load_dict(self._config, top=self)
            self._is_loaded = True

        def _validate(self):
            return True

        def to_json(self):
            #print("c", self._config)
            return json.dumps(self._config)

        def get(self, name, default_value=None):
            if hasattr(self, name):
                return getattr(self, name)
            else:
                return default_value

        @property
        def is_loaded(self):
            return self._is_loaded

        def _load_dict(self, d, **kwargs):
            top = kwargs.get("top", _KerviConfig())
            seqs = tuple, list, set, frozenset
            for i, j in d.items():
                top._keys += [i]
                if isinstance(j, dict):
                    item = self._load_dict(j)
                    setattr(top, i, item )
                elif isinstance(j, seqs):
                    item = type(j)(self._load_dict(sj) if isinstance(sj, dict) else sj for sj in j)
                    setattr(
                        top,
                        i,
                        item
                    )
                else:
                    setattr(top, i, j)
            #setattr(top, "keys", keys)

            return top

        def list(self):
            print("cl", dir(self))


    
    def __new__(cls): # __new__ always a classmethod
        if not _Configuration.instance:
            _Configuration.instance = _Configuration.__ConfigClass()
        return _Configuration.instance

    def __getattr__(self, name):
        if name == "instance":
            return _Configuration.instance
        return getattr(_Configuration.instance, name)

    def __setattr__(self, name, value):
        return setattr(_Configuration.instance, name, value)
